#!/usr/bin/python

import bzrlib.branch
from errors import CommandError
from bzrlib.option import Option
from patchsource import BzrPatchSource
from shelf import Shelf

class cmd_shelve(bzrlib.commands.Command):
    """Temporarily set aside some changes to the current working tree.

    Shelve allows you to temporarily put changes you've made "on the shelf",
    ie. out of the way, until a later time when you can bring them back from
    the shelf with the 'unshelve' command.

    You can put multiple items on the shelf, each time you run unshelve the
    most recently shelved changes will be reinstated.

    If filenames are specified, only the changes to those files will be
    shelved, other files will be left untouched.

    If a revision is specified, changes since that revision will be shelved.

    If you specifiy "--pick" you'll be prompted for each hunk of the diff as
    to whether you want to shelve it or not. Press "?" at the prompt for help.
    """
    takes_args = ['file*']
    takes_options = [Option('pick'), 'message', 'revision']
    def run(self, pick=False, file_list=None, message=None, revision=None):
        if revision is not None and revision:
            if len(revision) == 1:
                revision = revision[0]
            else:
                raise CommandError("shelve only accepts a single revision "
                                  "parameter.")

        source = BzrPatchSource(revision, file_list)
        s = Shelf(source.branch.base)
        s.shelve(source, pick, message)
        return 0

class cmd_unshelve(bzrlib.commands.Command):
    """Reinstate the most recently shelved changes.
    See 'shelve' for more information.
    """
    takes_options = [Option('pick')]
    def run(self, pick=False):
        source = BzrPatchSource()
        s = Shelf(source.branch.base)
        s.unshelve(source, pick)
        return 0

bzrlib.commands.register_command(cmd_shelve)
bzrlib.commands.register_command(cmd_unshelve)

def test_suite():
    from bzrlib.tests.TestUtil import TestLoader
    import tests
    return TestLoader().loadTestsFromModule(tests)
