#!/usr/bin/python

import os
import sys
from errors import CommandError
from hunk_selector import ShelveHunkSelector, UnshelveHunkSelector
from diffstat import DiffStat
from patchsource import PatchSource, FilePatchSource

BASE_DIR = '.shelf'

class Shelf(object):
    def __init__(self, base, name='default'):
        self.name = name
        self.base = base
        shelf_base = os.path.join(self.base, BASE_DIR)
        self.shelf_dir = os.path.join(shelf_base, name)

        for dir in [shelf_base, self.shelf_dir]:
            if not os.path.isdir(dir):
                os.mkdir(dir)

    def __path(self, idx):
        return os.path.join(self.shelf_dir, '%.2d' % idx)

    def next_shelf(self):
        index = 0
        while True:
            name = self.__path(index)
            if not os.path.exists(name):
                return name
            index += 1

    def last_shelf(self):
        shelves = os.listdir(self.shelf_dir)
        indexes = [int(f) for f in shelves]
        indexes.sort()

        if len(indexes) == 0:
            return None

        return self.__path(indexes[-1])

    def get_shelf_message(self, shelf):
        prefix = "# shelf: "
        if not shelf.startswith(prefix):
            return None
        return shelf[len(prefix):shelf.index('\n')]

    def __show_status(self, source):
        if source.can_live_update():
            diff_stat = str(DiffStat(source.readlines()))
            if len(diff_stat) > 0:
                print >> sys.stderr, 'Diff status is now:\n', diff_stat
            else:
                print >> sys.stderr, 'No changes left in working tree.'

    def unshelve(self, patch_source, pick_hunks=False):
        shelf = self.last_shelf()

        if shelf is None:
            raise CommandError("No shelf found in '%s'" % self.base)

        patches = FilePatchSource(shelf).readpatches()
        if pick_hunks:
            to_unshelve, to_remain = UnshelveHunkSelector(patches).select()
        else:
            to_unshelve = patches
            to_remain = []

        if len(to_unshelve) == 0:
            raise CommandError('Nothing to unshelve')

        print >>sys.stderr, "Reapplying shelved patches",
        message = self.get_shelf_message(open(shelf, 'r').read())
        if message is not None:
            print >>sys.stderr, ' "%s"' % message
        else:
            print >>sys.stderr, ""

        pipe = os.popen('patch -d %s -s -p0' % self.base, 'w')
        for patch in to_unshelve:
            pipe.write(str(patch))
        pipe.flush()

        if pipe.close() is not None:
            raise CommandError("Failed running patch!")

        if len(to_remain) == 0:
            os.remove(shelf)
        else:
            f = open(shelf, 'w')
            for patch in to_remain:
                f.write(str(patch))
            f.close()

        self.__show_status(patch_source)

    def shelve(self, patch_source, pick_hunks=False, message=None):
        patches = patch_source.readpatches()

        if pick_hunks:
            to_shelve = ShelveHunkSelector(patches).select()[0]
        else:
            to_shelve = patches

        if len(to_shelve) == 0:
            raise CommandError('Nothing to shelve')

        shelf = self.next_shelf()
        print >>sys.stderr, "Saving to shelf '%s', patch '%s'" % \
                (self.name, os.path.basename(shelf))
        shelf = open(shelf, 'a')
        if message is not None:
            assert '\n' not in message
            shelf.write("# shelf: %s\n" % message)
        for patch in to_shelve:
            shelf.write(str(patch))

        shelf.flush()
        os.fsync(shelf.fileno())
        shelf.close()

        print >>sys.stderr, "Reverting shelved patches."
        pipe = os.popen('patch -d %s -sR -p0' % self.base, 'w')
        for patch in to_shelve:
            pipe.write(str(patch))
        pipe.flush()

        if pipe.close() is not None:
            raise CommandError("Failed running patch!")

        self.__show_status(patch_source)
